package com.ruoyi.project.business.service.impl;

import java.io.File;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.enums.FileBuzType;
import com.ruoyi.common.enums.SerialNumberPrefix;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.MessageUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.office.doc.CommonDocUtils;
import com.ruoyi.framework.config.RuoYiConfig;
import com.ruoyi.framework.redis.RedisCache;
import com.ruoyi.project.business.domain.*;
import com.ruoyi.project.business.domain.bo.TqContractBo;
import com.ruoyi.project.business.domain.bo.contract.TqContractLiftBo;
import com.ruoyi.project.business.domain.flow.FlowAuditParam;
import com.ruoyi.project.business.domain.vo.contract.TqContractDto;
import com.ruoyi.project.business.domain.vo.contract.TqContractLiftVo;
import com.ruoyi.project.business.domain.vo.contract.TqContractVo;
import com.ruoyi.project.business.domain.vo.currency.ContractCurrencyTime;
import com.ruoyi.project.business.dto.TaskGenerateParams;
import com.ruoyi.project.business.enums.AuditFlowEnum;
import com.ruoyi.project.business.mapper.TbCountryMapper;
import com.ruoyi.project.business.mapper.TbFileMapper;
import com.ruoyi.project.business.mapper.contract.TqContractHistoryMapper;
import com.ruoyi.project.business.mapper.contract.TqContractLiftHistoryMapper;
import com.ruoyi.project.business.mapper.contract.TqContractLiftMapper;
import com.ruoyi.project.business.mapper.contract.TqContractMapper;
import com.ruoyi.project.business.service.ITbContractCurrencyService;
import com.ruoyi.project.business.service.ITbFileService;
import com.ruoyi.project.business.service.ITqContractHistoryService;
import com.ruoyi.project.business.service.flow.IFlowService;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.mapper.SysDictDataMapper;
import fr.opensagres.xdocreport.document.images.FileImageProvider;
import fr.opensagres.xdocreport.template.formatter.NullImageBehaviour;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.project.business.service.ITqContractService;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

import static com.ruoyi.project.business.enums.AuditFlowEnum.*;

/**
 * 合同列表Service业务层处理
 * 
 * @author ruoyi
 * @date 2024-08-13
 */
@Service
public class TqContractServiceImpl implements ITqContractService 
{
    @Resource
    private TqContractMapper tqContractMapper;
    @Resource
    private RedisCache redisCache;
    @Resource
    private TbFileMapper tbFileMapper;

    @Resource
    private TqContractLiftMapper tqContractLiftMapper;

    @Resource
    private ITbFileService tbFileService;

    @Resource
    private ITqContractHistoryService tqContractHistoryService;

    @Resource
    IFlowService flowService;

    @Resource
    SysDictDataMapper sysDictDataMapper;

    public static final String APPROVE_STATUS_SUCCESS = "2";

    public static final String GENERATE_TASKS_FLAG = "1";
    @Resource
    TqContractLiftHistoryMapper tqContractLiftHistoryMapper;

    @Resource
    TbCountryMapper tbCountryMapper;

    @Resource
    ITbContractCurrencyService tbContractCurrencyService;

    /**
     * 查询合同列表
     * 
     * @param id 合同列表主键
     * @return 合同列表
     */
    @Override
    public TqContract selectTqContractById(Long id)
    {
        return tqContractMapper.selectTqContractById(id);
    }

    public TqContractVo getContractById(Long id){
        TqContractBo tqContractBo = new TqContractBo();
        tqContractBo.setId(id);
        TqContractVo returnTqContractBo = tqContractMapper.selectTqContractBoList(tqContractBo).get(0);
        List<TbFile> annex = tbFileService.list(Wrappers.<TbFile>lambdaQuery().eq(TbFile::getBusinessId, id)
                .eq(TbFile::getBusinessType, FileBuzType.CONTRACT.getType()));
        if (Objects.nonNull(tqContractBo)) {
            returnTqContractBo.setFile(annex.stream().map(TbFile::getFilePath).collect(Collectors.joining(",")));
        } else {
            throw new ServiceException(MessageUtils.message("common.dataDeleted"));
        }
        return returnTqContractBo;

    }

    /**
     * 查询合同列表列表
     * 
     * @param tqContract 合同列表
     * @return 合同列表
     */
    @Override
    public List<TqContract> selectTqContractList(TqContract tqContract)
    {
        return tqContractMapper.selectTqContractList(tqContract);
    }

    public List<TqContractVo> selectTqContractBoList(TqContractBo tqContract){
        return tqContractMapper.selectTqContractBoList(tqContract);
    }

    /**
     * 新增合同列表
     * 
     * @param tqContractDto 合同列表
     * @return 结果
     */
    @Override
    @Transactional
    public int insertTqContract(TqContractDto tqContractDto)
    {
        //补充创建日期和权限
        tqContractDto.setCreateTime(DateUtils.getNowDate());
        tqContractDto.setPermissionChar(SecurityUtils.getPermissionCharIns());
        //获取国家缩写
        TbCountry tbCountry = tbCountryMapper.selectTbCountryByCountryId(tqContractDto.getCountryId());
        if(ObjectUtils.isEmpty(tbCountry)){
            throw new ServiceException("未找到该国家的信息！");
        }
        //将前缀和国家信息补充完整
        String contractNo = redisCache.generateSerialNumberNew(SerialNumberPrefix.C.getPrefix(), tbCountry.getCountryAbbr());
        tqContractDto.setContractNo(contractNo);
        //根据币种获取签单时币种汇率
        List<ContractCurrencyTime> contractCurrencyTimeList = new ArrayList<>();
        contractCurrencyTimeList.add(new ContractCurrencyTime(
                Long.valueOf(tqContractDto.getCurrencyId()),
                tqContractDto.getSigningDate()
        ));
        Map<Long, BigDecimal> idAndRateMap = tbContractCurrencyService.selectExchangeRateByCurrencyId(contractCurrencyTimeList);
        //将签单币种汇率赋值
        if(idAndRateMap.containsKey(Long.parseLong(tqContractDto.getCurrencyId()))){
            tqContractDto.setRate4SigningContracts(idAndRateMap.get((Long.parseLong(tqContractDto.getCurrencyId()))));
        }
        //将合同的
        tqContractMapper.insertTqContract(tqContractDto);
        //附件上传
        tbFileService.saveOrUpdateFile(tqContractDto.getFile(),tqContractDto.getId(),FileBuzType.CONTRACT.getType());
        // 梯形 子表   采用批量导入的形式
        List<TqContractLift> tqContractLiftList = tqContractDto.getTqContractLiftList();
        if(CollectionUtils.isNotEmpty(tqContractLiftList)){
            //补充完整信息
            tqContractLiftList = attachTqContractLift(tqContractDto,tqContractLiftList);
            tqContractLiftMapper.insertTqContractLiftBatch(tqContractLiftList);
        }
        //生成对应审批流
        this.generateTasks(tqContractDto);
        return 1;
    }

    /**
     * 修改合同列表
     * 
     * @param tqContractDto 合同列表
     * @return 结果
     */
    @Override
    @Transactional
    public int updateTqContract(TqContractDto tqContractDto)
    {
        List<TqContractLift> tqContractLiftList = tqContractDto.getTqContractLiftList();
        if(CollectionUtils.isNotEmpty(tqContractLiftList)){
            //获取设备更新List
            List<TqContractLift> tqContractLiftUpdateList = tqContractLiftList.stream().filter(item-> item.getId() != null).collect(Collectors.toList());
            List<Long> idList = tqContractLiftUpdateList.stream().map(item->item.getId()).collect(Collectors.toList());
            //删除当前合同下不存在的设备番号
            if(CollectionUtils.isNotEmpty(idList)){
                tqContractLiftMapper.deleteTqContractLiftByNoId(tqContractDto.getContractNo(),idList);
            }
            if(CollectionUtils.isNotEmpty(tqContractLiftUpdateList)){
                tqContractLiftUpdateList = attachTqContractLift(tqContractDto,tqContractLiftUpdateList);
                tqContractLiftMapper.updateTqContractLiftBatch(tqContractLiftUpdateList);
            }
            //获取设备新增List
            List<TqContractLift> tqContractLiftAddList = tqContractLiftList.stream().filter(item-> item.getId() == null).collect(Collectors.toList());
            if(CollectionUtils.isNotEmpty(tqContractLiftAddList)){
                tqContractLiftAddList = attachTqContractLift(tqContractDto,tqContractLiftAddList);
                tqContractLiftMapper.insertTqContractLiftBatch(tqContractLiftAddList);
            }
        }
        tqContractDto.setUpdateTime(DateUtils.getNowDate());
        if(tqContractDto.getNeedFileFlag() != null && "1".equals(tqContractDto.getNeedFileFlag())){
            //附件上传
            tbFileService.saveOrUpdateFile(tqContractDto.getFile(), tqContractDto.getId(),FileBuzType.CONTRACT.getType());
        }

        //附件上传完毕后  新增履历表(存在履历信息就上传履历表)
        if(ObjectUtils.isNotEmpty(tqContractDto.getTqContractHistory())){
            tqContractHistoryService.insertTqContractHistory(tqContractDto.getTqContractHistory());
        }
        //在合同变更或者编辑页面操作的   （设备的审批流要重新恢复到status的初始状态）audit_status
        this.generateTasks(tqContractDto);

        return tqContractMapper.updateTqContract(tqContractDto);
    }



    private void generateTasks(TqContractDto tqContractDto){
        //付款方式是信用证  || 付款比例不是是30/70  走财务流程
        AuditFlowEnum contract = null;
        contract = "1".equals(tqContractDto.getPaymentMethod()) || !"1".equals(tqContractDto.getPaymentRatio())? CONTRACT_FINANCE_YES : CONTRACT_FINANCE_NO;
        //判断是不是变更或者编辑界面操作的数据
        if(GENERATE_TASKS_FLAG.equals(tqContractDto.getGenerateTasksFlag())){
            flowService.generateTasks(
                    new TaskGenerateParams(contract, tqContractDto, SecurityUtils.getUserId(),
                            tqContractDto.getContractNo()));
        }
    }
    private void updateFlow(TqContract tqContract){
        FlowAuditParam taskGenerateParams = new FlowAuditParam();
        taskGenerateParams.setAuditFlow(3L);
        taskGenerateParams.setBusinessId(tqContract.getId());
        taskGenerateParams.setStatus(0);
        taskGenerateParams.setRemark("重新审批");
        flowService.audit(taskGenerateParams);
    }

     /**
      * @Author Chenj
      * @Description 将合同信息补充完整
      * @Date 9:13 2024/8/21
      **/
    private List<TqContractLift>  attachTqContractLift(TqContractDto tqContractDto, List<TqContractLift>  tqContractLiftList){
        //根据实际排产日期查询汇率
        List<ContractCurrencyTime> scheduleList = tqContractLiftList.stream()
                .filter(tqContractLift -> tqContractLift.getSigningDate() != null) // Filter out null signing dates
                .map(tqContractLift -> new ContractCurrencyTime(
                        Long.parseLong(tqContractLift.getCurrencyId()), // Assuming this returns a long
                        tqContractLift.getActualProductionScheduleDate()
                ))
                .collect(Collectors.toList());
        Map<Long, BigDecimal> scheduleListIdAndRateMap = tbContractCurrencyService.selectExchangeRateByCurrencyId(scheduleList);
        //根据实际发货排产日期查询汇率
        List<ContractCurrencyTime> deliveryList = tqContractLiftList.stream()
                .filter(tqContractLift -> tqContractLift.getSigningDate() != null) // Filter out null signing dates
                .map(tqContractLift -> new ContractCurrencyTime(
                        Long.parseLong(tqContractLift.getCurrencyId()), // Assuming this returns a long
                        tqContractLift.getActualDeliveryDate()
                ))
                .collect(Collectors.toList());
        Map<Long, BigDecimal> deliveryListIdAndRateMap = tbContractCurrencyService.selectExchangeRateByCurrencyId(deliveryList);

        //获取版本
        Long version = tqContractLiftHistoryMapper.selectVersionMaxFromHistory(tqContractDto.getContractNo());
        String contractNoVersionNo = version == null ? "1": String.valueOf(version+1L);
        tqContractLiftList.forEach(item -> {
            item.setContractNo(tqContractDto.getContractNo());
            item.setContractName(tqContractDto.getContractName());
            item.setAreaId(tqContractDto.getAreaId());
            item.setCountryId(tqContractDto.getCountryId());
            item.setCity(tqContractDto.getCity());
            item.setSalesman(tqContractDto.getSalesman());
            item.setSalesModel(tqContractDto.getSalesModel());
            item.setEndCustomers(tqContractDto.getEndCustomers());
            item.setTradeTerms(tqContractDto.getTradeTerms());
            item.setAgentId(tqContractDto.getAgentId());
            item.setPaymentMethod(tqContractDto.getPaymentMethod());
            item.setAmountOfMoney(tqContractDto.getAmount());
            item.setSigningDate(tqContractDto.getSigningDate());
            item.setContractAmendmentDate(new Date());
            item.setPaymentRatio(tqContractDto.getPaymentRatio());
            item.setContractNoVersionNo(tqContractDto.getContractNo()+"-"+contractNoVersionNo);
            item.setPermissionChar(SecurityUtils.getPermissionCharIns());
            item.setRate4SigningContracts(tqContractDto.getRate4SigningContracts());
            //将签单币种汇率赋值
            if(scheduleListIdAndRateMap.containsKey(Long.parseLong(item.getCurrencyId()))){
                item.setRate4ProductionScheduling(scheduleListIdAndRateMap.get(Long.parseLong(item.getCurrencyId())));
            }
            if(deliveryListIdAndRateMap.containsKey(Long.parseLong(item.getCurrencyId()))){
                item.setRate4Shipment(deliveryListIdAndRateMap.get(Long.parseLong(item.getCurrencyId())));
            }
        });
        return tqContractLiftList;
    }



    /**
     * 批量删除合同列表
     * 
     * @param ids 需要删除的合同列表主键
     * @return 结果
     */
    @Override
    public int deleteTqContractByIds(Long[] ids)
    {
        return tqContractMapper.deleteTqContractByIds(ids);
    }

    /**
     * 删除合同列表信息
     * 
     * @param id 合同列表主键
     * @return 结果
     */
    @Override
    public int deleteTqContractById(Long id)
    {
        return tqContractMapper.deleteTqContractById(id);
    }

    @Transactional
    @Override
    public void exportContract(Long id){
        //1.获取基础数据
        //1.1合同
        TqContract tqContract = this.getContractById(id);
        List<JSONObject> contractLift = Lists.newArrayList();
        //1.2电梯
        TqContractLiftBo tqContractLiftBo = new TqContractLiftBo();
        tqContractLiftBo.setContractNo(tqContract.getContractNo());
        List<TqContractLiftVo> tqContractLiftList = tqContractLiftMapper.selectTqContractLiftList(tqContractLiftBo);
        //货币单位
        SysDictData sysDictData =  sysDictDataMapper.selectSysDictDataByTypeAndValue("currency_type",tqContract.getCurrencyId());
        if(ObjectUtils.isEmpty(sysDictData) && StringUtils.isEmpty(sysDictData.getRemark())){
            throw new ServiceException("未找到该合同的单位");
        }
        String currencyUnit =  String.valueOf(sysDictData.getRemark().charAt(sysDictData.getRemark().length() - 1));

        // 2. 转换数据格式
        int i = 1;
        BigDecimal totalPrice = new BigDecimal(BigDecimal.ROUND_UP);
        for (TqContractLiftVo item : tqContractLiftList) {
            JSONObject data = new JSONObject();
            data.put("index",i);
            data.put("elevatorSpecifie", item.getProjectName()+item.getSpecification());
            data.put("unitPrice",currencyUnit + item.getUnitPrice());
            data.put("count",item.getCount());
            data.put("elvPrice",currencyUnit + item.getUnitPrice().multiply(BigDecimal.valueOf(item.getCount())));
            totalPrice  = totalPrice.add(item.getUnitPrice().multiply(BigDecimal.valueOf(item.getCount())));
            contractLift.add(data);
            i++;
        }
        Map<String, Object> map = Maps.newHashMap();
        map.put("buyers",tqContract.getContractName());
        map.put("address",tqContract.getAddress());
        map.put("telephone",tqContract.getShippingMark());
        map.put("totalPrice",currencyUnit +totalPrice);
        map.put("email",tqContract.getEmail());
        map.put("destinationPort",tqContract.getDestinationPort());
        map.put("shippingMark",tqContract.getShippingMark());
        map.put("shipmentPort",tqContract.getShipmentPort());
        map.put("shipmentDeadline", DateUtils.parseDateToStr("yyyy-MM-dd", tqContract.getShipmentDeadline()));
        map.put("contractLift", contractLift);


//        String path = CollectionUtils.isNotEmpty(previewList) ? RuoYiConfig.getProfile() + StringUtils.substringAfter(previewList.get(0).getFilePath(), Constants.RESOURCE_PREFIX): "/null";
//        FileImageProvider file = new FileImageProvider(new File(path), true);
        //只有审核通过的才会盖章
        if(APPROVE_STATUS_SUCCESS.equals(tqContract.getApproveStatus())){
            String path = "src/main/resources/wordTemplate/章.png";
            FileImageProvider file = new FileImageProvider(new File(path), true);
            file.setBehaviour(NullImageBehaviour.RemoveImageTemplate);
            file.setWidth(100f);
            file.setResize(true);
            map.put("photoSeal", file);
        }

        // 3. 填充模板并返回
        new CommonDocUtils()
                .loadFromClasspath("/wordTemplate/国际部销售合同模版2024.docx")
                .replaceData(map)
                .downloadDocx();

    }
}
