package com.ruoyi.project.business.service.quotation.impl;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.WordUtil;
import com.ruoyi.common.utils.office.doc.CommonDocUtils;
import com.ruoyi.framework.config.RuoYiConfig;
import com.ruoyi.project.business.domain.TqBargainPrice;
import com.ruoyi.project.business.domain.TqQuotation;
import com.ruoyi.project.business.domain.bo.quotation.QuotationExportBO;
import com.ruoyi.project.business.domain.vo.quotation.ElevatorExportVO;
import com.ruoyi.project.business.domain.vo.quotation.ElevatorParamExportVO;
import com.ruoyi.project.business.domain.vo.quotation.QuotationExportVO;
import com.ruoyi.project.business.domain.vo.quotation.WParam;
import com.ruoyi.project.business.mapper.quotation.TqQuotationMapper;
import com.ruoyi.project.business.service.ITbCurrencyService;
import com.ruoyi.project.business.service.ITcProjectService;
import com.ruoyi.project.business.service.quotation.IQuotationConstantService;
import com.ruoyi.project.business.service.quotation.IQuotationExportService;
import com.ruoyi.project.business.service.quotation.ITqBargainPriceService;
import com.ruoyi.project.business.service.quotation.ITqSpecificationService;
import fr.opensagres.xdocreport.document.images.FileImageProvider;
import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

@Service
@Slf4j
@RequiredArgsConstructor
public class QuotationExportServiceImpl
    extends ServiceImpl<TqQuotationMapper, TqQuotation> implements IQuotationExportService, IQuotationConstantService {

    public final ITcProjectService tcProjectService;

    public final ITbCurrencyService tbCurrencyService;

    public final ITqBargainPriceService tqBargainPriceService;

    public final ITqSpecificationService specificationService;

    @Override
    public void exportQuotation(HttpServletResponse response, QuotationExportBO quotationExportBO) throws IOException {
        List<Long> quotationIdList = quotationExportBO.getQuotationIdList();
        // 报价数据
        List<QuotationExportVO> quotationExportList = this.baseMapper.listQuotationExport(quotationIdList);
        Map<Long, List<QuotationExportVO>> quotationMap = quotationExportList.stream()
            .collect(Collectors.groupingBy(QuotationExportVO::getQuotationId));
        // 电梯数据
        List<ElevatorExportVO> specificationList = specificationService.listElevatorExport(quotationIdList);
        Map<Long, List<ElevatorExportVO>> elevatorMap = specificationList.stream()
            .collect(Collectors.groupingBy(ElevatorExportVO::getQuotationId));

        // 多个报价单文件
        List<File> fileList = new ArrayList<>();
        for (Long quotationId : quotationIdList) {
            List<QuotationExportVO> tmpList = quotationMap.getOrDefault(quotationId, new ArrayList<>());
            QuotationExportVO quotation = CollUtil.isNotEmpty(tmpList) ? tmpList.get(0) : new QuotationExportVO();

            List<ElevatorExportVO> elevatorDataList = elevatorMap.get(quotationId);

            Map<String, Object> map = new HashMap<>();
            // 首页
            map.put(QUOTATION_NO, getStr(quotation.getQuotationNo()));
            // 项目
            map.put(PROJECT, getStr(quotation.getProjectName()));
            // 地址
            map.put(ADDRESS, getStr(quotation.getAddress()));
            // 客户
            map.put(CLIENT, getStr(quotation.getClient()));
            // 邮箱
            map.put(EMAIL, getStr(quotation.getEmail()));
            // 日期
            map.put(DATE, getStr(DateUtils.parseDateToStr(MM_DD_YYYY, quotation.getSubmitDate())));
            // 运费
            map.put(FREIGHT_PRICE, formatNum(quotation.getFreightPrice()));
            // 总价
            map.put(PRICE_TOTAL, formatNum(quotation.getFinalPrice()));
            // 币种汇率
            BigDecimal divide1 = BigDecimal.ONE.divide(quotation.getExchangeRate(), 2, RoundingMode.HALF_UP);
            map.put(CURRENCY, formatNum2(divide1));

            // TODO: 2024/7/31
            // 目的地
            map.put(DESTINATION, "unknown");
            // 集装箱信息
            map.put(CONTAINER, "unknown");

            // 议价数据（折扣）
            // 整单折扣没存，需要算
            map.put(DISCOUNT, "0%");
            List<TqBargainPrice> list = tqBargainPriceService.list(
                Wrappers.<TqBargainPrice>lambdaQuery().eq(TqBargainPrice::getQuotationId, quotationId)
                    .orderByDesc(TqBargainPrice::getId));
            if (CollUtil.isNotEmpty(list)) {
                TqBargainPrice tqBargainPrice = list.get(0);
                BigDecimal discountBasicPrice = tqBargainPrice.getAcceptPrice().subtract(quotation.getOptionalPrice());
                BigDecimal divide = discountBasicPrice.divide(quotation.getOriginalBasicPrice(), 2, RoundingMode.DOWN);
                map.put(DISCOUNT, formatNum(divide.multiply(new BigDecimal(100))) + "%");
            }

            // 电梯价格表
            for (ElevatorExportVO elevator : elevatorDataList) {
                // 电梯规格单价
                map.put(UNIT_PRICE, formatNum(elevator.getPrice()));
                // 电梯规格数量
                map.put(QTY, elevator.getElevatorNumber());
                // 电梯规格总价
                BigDecimal multiply = elevator.getPrice().multiply(new BigDecimal(elevator.getElevatorNumber()));
                map.put(T_PRICE, formatNum(multiply));

                // 梯型参数MAP  KEY:参数编码  VALUE:参数对象
                Map<String, ElevatorParamExportVO> paramMap = new HashMap<>();
                // 梯型参数MAP  KEY:参数编码  VALUE:参数值
                Map<String, String> paramCodeValueMap = new HashMap<>();
                List<ElevatorParamExportVO> paramList = elevator.getParamList();
                if (CollUtil.isNotEmpty(paramList)) {
                    paramMap = paramList.stream()
                        .collect(Collectors.toMap(ElevatorParamExportVO::getParaCode, a -> a, (a, b) -> a));
                    paramCodeValueMap = paramList.stream()
                        .filter(a -> StringUtils.isNotBlank(a.getParaCode()))
                        .filter(a -> StringUtils.isNotBlank(a.getParaValue()))
                        .collect(
                            Collectors.toMap(ElevatorParamExportVO::getParaCode, ElevatorParamExportVO::getParaValue,
                                (a, b) -> a));
                }

                map.put(SPECIFICATION, getStr(elevator.getSpecification()));
                map.put(PRODUCT_TYPE, getStr("unknown"));
                map.put(STAND, getStr("unknown"));
                String rateq = paramCodeValueMap.get("RATEQ");
                map.put(LOAD, getStr(rateq));
                String p = "";
                try {
                    p = String.valueOf(Long.parseLong(rateq) / 75);
                } catch (Exception e) {
                    // ignore
                }
                map.put(P, getStr(p));
                map.put(SPEED, getStr(paramCodeValueMap.get("SPEED")));
                map.put(FLOORS, getStr(paramCodeValueMap.get("LANDS")));
                map.put(STOPS, getStr(paramCodeValueMap.get("STOPS")));
                map.put(DOORS, getStr(paramCodeValueMap.get("DOORS")));
                String through = paramCodeValueMap.get("THROUGH");
                // 贯通：2  不贯通：1
                String throughStr = "2";
                if ("No".equals(through)) {
                    throughStr = "1";
                }
                map.put(MAIN_FLOOR, getStr(throughStr));
                String cw = paramCodeValueMap.get("CW");
                String ch = paramCodeValueMap.get("CH");
                String cd = paramCodeValueMap.get("CD");
                map.put(CABIN, getStr(cw) + "*" + getStr(ch) + "*" + getStr(cd));
                String op = paramCodeValueMap.get("OP");
                String oph = paramCodeValueMap.get("OPH");
                map.put(DOOR, getStr(op) + "*" + getStr(oph));
                String hw = paramCodeValueMap.get("HW");
                String hd = paramCodeValueMap.get("HD");
                map.put(SHAFT, getStr(hw) + "*" + getStr(hd));
                map.put(PIT, getStr(paramCodeValueMap.get("PD")));
                map.put(OH, getStr(paramCodeValueMap.get("OH")));
                map.put(TR, getStr(paramCodeValueMap.get("TR")));

                // 测试图片
                Map<String, Object> picMap = new HashMap<>();
                String path = "C:\\Users\\jinyw\\Desktop\\koyo\\白狐.png";
                picMap.put("test111", new FileImageProvider(new File(path)));

                // 电梯参数
                File file = new CommonDocUtils()
                    .loadFromClasspath("/wordTemplate/KW2600模版.docx")
                    .replaceData(map)
//                    .replacePicData(picMap)
                    .genFile(getStr(quotation.getQuotationNo()));
                fileList.add(file);
            }

        }
        WordUtil.exportZip(response, fileList);
    }

    private static String getStr(String str) {
        return StrUtil.isBlank(str) ? " " : str;
    }

    private static String formatNum(Object decimal) {
        return Objects.isNull(decimal) ? " " : format.format(decimal);
    }

    private static String formatNum2(Object decimal) {
        return Objects.isNull(decimal) ? " " : format2.format(decimal);
    }

    private static String getPicBase64(String path) {
        String encode = "";
        if (StringUtils.isBlank(path)) {
            log.info("[报价导出]图片文件地址为空");
            return encode;
        }
        String pathSuffix = path.substring(Math.min(8, path.length() - 1));
        String absolutePath = new File(RuoYiConfig.getProfile()).getAbsolutePath() + pathSuffix;
        File file = FileUtil.file(absolutePath);
        if (file.exists()) {
            try {
                encode = Base64.encode(FileUtil.readBytes(file));
            } catch (Exception e) {
                log.error("[报价导出]读取/转码异常:", e);
            }
        } else {
            log.info("[报价导出]图片文件不存在:[{}]", absolutePath);
        }
        return encode;
    }

    private static Map<String, Object> getMap(ElevatorExportVO elevator) {
        Map<String, Object> firstElevator = new HashMap<>();
        List<WParam> baseParamList = new ArrayList<>();
        baseParamList.add(new WParam("额定载重", "10KG"));
        baseParamList.add(new WParam("额定速度", "10m/s"));
        firstElevator.put("baseParam", baseParamList);
        return firstElevator;
    }
}
